using System;
using System.Collections.Generic;
using System.Text;

namespace FBPLib
{
    /**
  * Look after input to subnet - added for subnet support
  * Substream-Sensitive SubIn
  */
    [InPort("NAME")]
    [OutPort("OUT")]
    public class SubInSS : Component
    {
        /* *
         * Copyright 2007, 2008, J. Paul Morrison.  At your option, you may copy, 
         * distribute, or make derivative works under the terms of the Clarified Artistic License, 
         * based on the Everything Development Company's Artistic License.  A document describing 
         * this License may be found at http://www.jpaulmorrison.com/fbp/artistic2.htm. 
         * THERE IS NO WARRANTY; USE THIS PRODUCT AT YOUR OWN RISK.
         * */
        IInputPort inport, nameport;

        OutputPort outport;

        public override void Execute()
        {

            Packet np = nameport.Receive();
            if (np == null)
            {
                return;
            }
            nameport.Close();
            string pname = np.Content as string;
            Drop(np);

            inport = (_mother._inputPorts)[pname] as IInputPort;
            Packet p;
            // I think this works!
            Component oldReceiver = inport.Receiver;
            if (inport is InitializationConnection)
                FlowError.Complain("SubinSS cannot support IIP - use Subin");
            Connection cnxt = inport as Connection;
            cnxt.Receiver = this;
            while ((p = cnxt.Receive()) != null)
            {
                p.Owner = this;
                if (p.Type == Packet.Types.Open)
                {
                    Drop(p);
                    _compLog.Trace("{0}: Open bracket detected", Name);
                }
                else if (p.Type == Packet.Types.Close)
                {
                    Drop(p);
                    _compLog.Trace("{0}: Close bracket detected", Name);
                    break;
                }
                else
                outport.Send(p);
            }


            // inport.close();

            inport.Receiver = oldReceiver;

            // inport = null;
        }

        public override Object[] Introspect()
        {
            return new Object[] { "handles one input substream for subnet", "NAME",
				"input", Type.GetType("System.String"), "name of higher level input port",
				"OUT", "output", Type.GetType("System.Object"),
				"output from external port (into subnet)" };
        }

        public override void OpenPorts()
        {

            nameport = OpenInput("NAME");
            outport = OpenOutput("OUT");
        }
    }

}
